/************************************
  REVISION LOG ENTRY
  Revision By: Mike ( mike@exontrol.com )
  Revised on 7/16/99 4:18:49 PM
  Comments: MaskWord.cpp: implementation of the CMaskWord class.

  This code may be used in compiled form in any way you desire. This
  file may not be redistributed modified or unmodified without the authors
  written consent.

 ************************************/

#include "stdafx.h"
#include "MaskWord.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

// Function name	: CMaskWord::CMaskWord
// Description	    : default constructor
// Return type		: 
CMaskWord::CMaskWord() : m_nPreviousState(Begin)
{
	m_nCurrentState = Undefined;
	m_nPatternSearch = 0;
}

// Function name	: CMaskWord::State
// Description	    : 
// Return type		: _StateEnum 
// Argument         : TCHAR c
CMaskWord::_StateEnum CMaskWord::State(TCHAR c)
{
	switch (c)
	{
		case '*': {
			return Any;
			break;
		}
		case '#': {
			return Digit;
			break;
		}
		case '?': {
			return One;
			break;
		}
	}
	return Char;
}


// Function name	: CMaskWord::CMaskWord
// Description	    : 
// Return type		: 
// Argument         : LPCTSTR szMaskWord
CMaskWord::CMaskWord( LPCTSTR szMaskWord ) : m_nPreviousState(Begin)
{
	Set( szMaskWord );
}

// Function name	: CMaskWord::~CMaskWord
// Description	    : virtual destructor
// Return type		: 
CMaskWord::~CMaskWord()
{

}

// Function name	: CMaskWord::Match
// Description	    : checks if the word is matched to one of the contained patterns
// Return type		: BOOL 
// Argument         : LPCTSTR szMatchWord
BOOL CMaskWord::Match( LPCTSTR szMatchWord )
{
	string str = "";
	if ( GetFirstPattern( str ) )
	{
		do
		{
			assign( str );
			if ( MatchWord( szMatchWord ) )
				return TRUE;
		} while ( GetNextPattern( str ) );
	}
	return FALSE;
}

// Function name	: CMaskWord::GetFirstPattern
// Description	    : Retrieves the first pattern
// Return type		: BOOL 
// Argument         : string& str
BOOL CMaskWord::GetFirstPattern( string& str )
{
	m_nPatternSearch = 0;
	return GetNextPattern( str );
}

// Function name	: CMaskWord::GetNextPattern
// Description	    : Retrieves the next pattern
// Return type		: BOOL 
// Argument         : string& str
BOOL CMaskWord::GetNextPattern( string& str )
{
	str = "";
	int l = m_strPatterns.length();
	for ( ; ( m_nPatternSearch < l ) && ( m_strPatterns[ m_nPatternSearch ] == ';' ); m_nPatternSearch++ );
	if ( m_nPatternSearch >= l ) 
		return FALSE;
	for ( ; ( m_nPatternSearch < l ) && ( m_strPatterns[ m_nPatternSearch ] != ';' ); m_nPatternSearch++ )
		str += m_strPatterns[ m_nPatternSearch ];
	return TRUE;
}

// Function name	: Match
// Description	    : Return TRUE if the szWord match with mask
// Return type		: BOOL 
// Argument         : LPCTSTR szWord
BOOL CMaskWord::MatchWord( LPCTSTR szMatchWord )
{
	m_nPreviousState = Begin;
	int nCountThis = length();
	string strLowerWord(szMatchWord);
	MakeLower( &strLowerWord );
	LPCTSTR szWord = strLowerWord.c_str();
	int nCrtWord = 0, nCountWord = strlen(szWord), nCrt = nCrtWord, nCrtThisStart = nCrtWord;
	BOOL bContinue = TRUE;
	while ( nCrt < nCountWord ) 
	{
		for ( int nCrtThis = nCrtThisStart; (bContinue) && (nCrtThis < nCountThis) && (nCrt < nCountWord); nCrtThis++ )
		{
			m_nCurrentState = State( at( nCrtThis ) );
			switch ( m_nCurrentState )
			{
				case One:
				{
					if ( m_nPreviousState == Begin )
						nCrtThisStart++;
					nCrt++;
					break;
				}
				case Digit:
				case Any:
				{
					// do nothing
					break;
				}
				case Char:
				{
					switch ( m_nPreviousState )
					{
						case Begin:
						case Char:
						case One:
						{
							//Match exacly by position
							if ( at(nCrtThis) != szWord[nCrt] )
							{
								if ( m_nPreviousState == Begin )
									return FALSE;
								bContinue = FALSE;
								break;
							}
							if ( m_nPreviousState == Begin )
								nCrtThisStart++;
							nCrt++;
							break;
						}
						case Any:
						{
							TCHAR* pCrt = strchr( szWord + nCrt, at(nCrtThis) );
							if ( pCrt == NULL )
								return FALSE;
							nCrt += pCrt - (szWord + nCrt);
							nCrt++;
							// Find first at(nCrtThis) into szWord + nCrt
							break;
						}
						case Digit:
						{

							TCHAR* pCrt = strchr( szWord + nCrt, at(nCrtThis) );
							if ( pCrt == NULL )
								return FALSE;

							long c = nCrt + ( pCrt - (szWord + nCrt) );
							for ( ; nCrt < c; nCrt++ )
								if ( !isdigit( *(LPCTSTR)(szWord + nCrt) ) )
								{
									bContinue = FALSE;
									break;
								}
							break;
						}
						default:
						{
							// I don't believe this. How can this be happen??!
							_ASSERTE( FALSE );
						}
					}
				}

			}
			m_nPreviousState = m_nCurrentState;
		}
		switch (m_nCurrentState)
		{
			case Any:
			{
				return TRUE;
				break;
			}
			case Digit:
			{
				if ( bContinue) 
				{
					BOOL bBreak = FALSE;
					for ( int i = nCrt; !bBreak && *(LPCTSTR)(szWord + i); i++ )
					{
						if ( !isdigit( *(LPCTSTR)(szWord + i) ) )
						{
							nCrt = ++nCrtWord;
							m_nPreviousState = Begin;
							bBreak = TRUE;
						}
					}
					if ( !bBreak )
						return TRUE;
				}
				break;
			}
			default:
			{
				if ( bContinue) 
					if ( ( nCrt == nCountWord )  && (nCrtThis == nCountThis) ) 
						return TRUE;
				nCrt = ++nCrtWord;
				m_nPreviousState = Begin;
			}
		}
		bContinue = TRUE;
	}
	return FALSE;
}

// Function name	: CMaskWord::MakeLower
// Description	    : makes the pSource string lower case
// Return type		: void 
// Argument         : string *pSource
void CMaskWord::MakeLower(string *pSource)
{
	_ASSERTE( pSource );
	for ( int i = 0; i < pSource->length(); i++ )
		if ( ( pSource->at( i ) >= 'A' ) && ( pSource->at( i ) <= 'Z' ) )
			pSource->at( i ) = pSource->at( i ) | 32;
}

// Function name	: CMaskWord::ExtractFileName
// Description	    : Extracts the file name from the szFileName.
// Return type		: string 
// Argument         : LPCTSTR szFileName
string CMaskWord::ExtractFileName(LPCTSTR szFileName)
{
	int l = strlen( szFileName );
	string strFileName = "";
	for ( int i = l - 1; i>= 0; i-- )
	{
		if ( szFileName[i] == '\\' )
			return strFileName;
		strFileName = szFileName[i] + strFileName;
	}
	return strFileName;
}

// Function name	: CMaskWord::Set
// Description	    : Sets the mask word
// Return type		: void 
// Argument         : LPCTSTR pszMaskWords
void CMaskWord::Set(LPCTSTR pszMaskWords)
{
	m_nCurrentState = Undefined;
	if ( pszMaskWords )
		m_strPatterns = pszMaskWords;
	MakeLower( &m_strPatterns );
	m_nPatternSearch = 0;
}
